angular.module('manageCampaignsApp', [
    'ngRoute',
    'manageCampaigns',
    'manageProfiles',
    'manageGoals',
    'wemServices',
    'wemDirectives'
])
    .config(['$routeProvider',
        function($routeProvider) {
            $routeProvider.
                when('/campaigns', {
                    templateUrl: ManagersContext.localTemplatesPath + '/campaigns/campaign-list.html',
                    controller: 'ManageCampaignsListCtrl'

                }).
                when('/newCampaign', {
                    templateUrl: ManagersContext.localTemplatesPath + '/campaigns/campaign-edit.html',
                    controller: 'ManageCampaignCtrl',
                    resolve: {
                        "currentCampaign": function(campaignService) {
                            var zonename = jstz.determine().name();

                            //The new campaign
                            if(campaignService.campaignToBeSaved) {
                                var currentCampaign = angular.copy(campaignService.campaignToBeSaved);
                                delete campaignService.campaignToBeSaved;
                                return currentCampaign;
                            } else {
                                return {
                                    metadata: {
                                        id: ManagersUtils.generateId(),
                                        name: "",
                                        description: "",
                                        scope: ManagersContext.dmfScope,
                                        tags: ["referrer"]
                                    },
                                    startDate: moment().tz(zonename).toDate().toISOString(),
                                    endDate: moment().tz(zonename).add(1, 'M').toDate().toISOString(),
                                    cost: 0,
                                    currency: "USD",
                                    timezone: zonename
                                };
                            }
                        },
                        "isNew": function() {
                            return true;
                        }
                    }
                }).
                when('/campaign/:campaignId', {
                    templateUrl: ManagersContext.localTemplatesPath + '/campaigns/campaign-edit.html',
                    controller: 'ManageCampaignCtrl',
                    resolve: {
                        // resolve rule here to avoid async issue using deferred API
                        "currentCampaign": function( $q, $route, campaignService) {
                            var campaign = $q.defer();
                            campaignService.getCampaign($route.current.params.campaignId).success(function(campaignData){
                                campaign.resolve(campaignData);
                            });
                            return campaign.promise;
                        },
                        "isNew": function() {
                            return false;
                        }
                    }
                }).
                when('/campaign/:campaignId/newGoal', {
                    templateUrl: ManagersContext.localTemplatesPath + '/goals/goal-create.html',
                    controller: 'CreateCampaignGoalCtrl',
                    resolve: {
                        "currentCampaign": function( $q, $route, campaignService) {
                            if (campaignService.campaignToBeSaved && campaignService.campaignToBeSaved.metadata.id == $route.current.params.campaignId) {
                                return campaignService.campaignToBeSaved;
                            }
                            var campaign = $q.defer();
                            console.log("Editing campaign:");
                            console.log(campaignService.getCampaign($route.current.params.campaignId));
                            campaignService.getCampaign($route.current.params.campaignId).success(function(campaignData){
                                campaign.resolve(campaignData);
                            });
                            return campaign.promise;
                        }
                    }
                }).
                when('/campaigndashboard/:campaignId', {
                    templateUrl: ManagersContext.localTemplatesPath + '/campaigns/campaign-goals-list.html',
                    controller: 'ManageCampaignGoalsListCtrl',
                    resolve: {
                        // resolve rule here to avoid async issue using deferred API
                        "campaignDetailed": function( $q, $route, campaignService) {
                            var campaign = $q.defer();
                            campaignService.getCampaignDetail($route.current.params.campaignId).success(function(campaignData){
                                if (campaignService.updatedCampaign) {
                                    campaignData.campaign = campaignService.updatedCampaign;
                                    delete campaignService.updatedCampaign;
                                }
                                campaign.resolve(campaignData);
                            });
                            return campaign.promise;
                        }
                    }
                }).
                when('/goal/:goalId', {
                    templateUrl: ManagersContext.localTemplatesPath + '/goals/goal-edit.html',
                    controller: 'ManageGoalCtrl',
                    resolve: {
                        // resolve goal here to avoid async issue using deferred API
                        "goal": function( $q, $route, goalService) {
                            var goal = $q.defer();
                            goalService.getGoal($route.current.params.goalId).success(function(goalData){
                                goal.resolve(goalData);
                            });
                            return goal.promise;
                        }
                    }
                }).
                when('/goal-report/:goalId', {
                    templateUrl: ManagersContext.localTemplatesPath + '/goals/goal-report.html',
                    controller: 'ManageGoalReportCtrl',
                    resolve: {
                        // resolve goal here to avoid async issue using deferred API
                        "goal": function ($q, $route, goalService) {
                            var goal = $q.defer();
                            goalService.getGoal($route.current.params.goalId).success(function (goalData) {
                                goal.resolve(goalData);
                            });
                            return goal.promise;
                        }
                    }
                }).
                when('/goal-report/:goalId/primary', {
                    templateUrl: ManagersContext.localTemplatesPath + '/goals/goal-report.html',
                    controller: 'ManageGoalReportCtrl',
                    resolve: {
                        // resolve goal here to avoid async issue using deferred API
                        "goal": function ($q, $route, goalService) {
                            var goal = $q.defer();
                            goalService.getGoal($route.current.params.goalId).success(function (goalData) {
                                goalData.isPrimary = true;
                                goal.resolve(goalData);
                            });
                            return goal.promise;
                        }
                    }
                }).
                when('/campaign/:campaignId/profiles', {
                    templateUrl: ManagersContext.localTemplatesPath + '/campaigns/profiles-list.html'
                }).
                when('/campaign/:campaignId/profile/:profileId', {
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/profile-view.html',
                    controller: 'ManageProfileCtrl',
                    resolve: {
                        // resolve profile here to avoid async issue using deferred API
                        "profile": function ($q, $route, profileService) {
                            var profile = $q.defer();
                            profileService.getProfile($route.current.params.profileId).success(function (profileData) {
                                profile.resolve(profileData);
                            });
                            return profile.promise;
                        }
                    }
                }).
                when('/campaign/:campaignId/profile/:profileId/:sessionId', {
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/profile-session.html',
                    controller: 'ManageSessionCtrl'
                }).
                otherwise({
                    redirectTo: '/campaigns'
                });
        }])
;