angular.module('internalSearches', ['wemServices', 'wemDirectives', 'ui.bootstrap', 'i18n', 'underscore'])
    .run(['$rootScope', function ($rootScope) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.pagerUrl = ManagersContext.localTemplatesPath + "/dt-pager.html";
    }])

    .controller('InternalSearchesListCtrl', ['$scope', 'searches', 'DTInstances', 'DTOptionsBuilder', 'DTColumnBuilder', '$compile', 'i18nService', '$location', 'internalSearchService', '_', function ($scope, searches, DTInstances, DTOptionsBuilder, DTColumnBuilder, $compile, i18nService, $location, internalSearchService, _) {
        $scope.searches = [];
        for (var search in searches) {
            if (ManagersUtils.acceptProperty(searches, search)) {
                $scope.searches.push({
                    searchString: search,
                    occurrences: searches[search],
                    importance: ManagersUtils.getPercentage(searches[search] / searches._filtered)
                });
            }
        }

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var orderData = _.find(aoData, function (x) {
                return x.name == "order"
            }).value;

            var order = _.map(orderData, function (o) {
                var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                return {prop: prop, dir: o.dir};
            })[0];
            var data = [];
            var count = 0;
            if (order) {
                $scope.searches.sort(function (a, b) {
                    if (order.prop == "searchString") {
                        return order.dir == 'asc' ? a[order.prop].localeCompare(b[order.prop]) : b[order.prop].localeCompare(a[order.prop]);
                    } else {
                        if (a[order.prop] > b[order.prop])
                            return order.dir == 'asc' ? 1 : -1;
                        if (a[order.prop] < b[order.prop])
                            return order.dir == 'asc' ? -1 : 1;
                    }
                    // a doit être égale à b
                    return 0;
                });
            }
            $scope.searches.forEach(function (val, index) {
                if (index >= start && count < length) {
                    val.index = count;
                    data.push(val);
                    count++;
                }
            });
            var results = {
                "sEcho": draw,
                "iTotalRecords": $scope.searches.length,
                "iTotalDisplayRecords": $scope.searches.length,
                "aaData": data
            };
            fnCallback(results);
            if ($scope.dtInstance) {
                ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                // need to call $scope.$digest to force variable link of the pager when reordering.
                if (!$scope.$$phase) {
                    $scope.$digest();
                }
            }
        }

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        function createdRow(row, data, dataIndex) {
            console.log("Row is created: " + row);
            // Recompiling so we can bind Angular directive to the DT
            $compile(angular.element(row).contents())($scope);
        }

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 'lt')
            .withFnServerData(serverData)
            .withOption('createdRow', createdRow)
            .withOption('order', [[2, 'desc']]);

        $scope.goToDetails = function (index) {
            internalSearchService.selectSearch($scope.dtInstance.DataTable.row(index).data().searchString);
            // the redirect to search details view using a fake route parameter so that we can properly find the previous page in profiles controllers
            $location.path('/search/selected');
        };

        function actionsHtml(data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.internalSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + data.index + ")\"><span class=\"md md-searches\"></span></button> ";
        }

        $scope.dtColumns = [
            DTColumnBuilder.newColumn("searchString").withTitle(i18nService.message("serverSettings.internalSearches.searchString")),
            DTColumnBuilder.newColumn("occurrences").withTitle(i18nService.message("serverSettings.internalSearches.occurrences")),
            DTColumnBuilder.newColumn("importance").withTitle(i18nService.message("serverSettings.internalSearches.importance")),
            DTColumnBuilder.newColumn(null).withTitle(i18nService.message("serverSettings.manageWem.listAction")).notSortable()
                .renderWith(actionsHtml).withOption("width", "180px").withOption("contentPadding", "0")
        ];
    }])

    .controller('InternalSearchesCtrl', ['$scope', '$routeParams', 'internalSearchService', 'i18nService', '$location', 'notificationService',
        function ($scope, $routeParams, internalSearchService, i18nService, $location, notificationService) {

            $scope.$on("$routeChangeError", function (evt, current, previous, rejection) {
                if (rejection == "profile_not_found") {
                    notificationService.error(i18nService.message("route.reject.profile_not_found"))
                }
            });

            $scope.searchDetail = internalSearchService.getSearchDetails();

            $scope.getTabTitle = function (key) {
                return i18nService.message(key);
            }
        }]);
