var ManagersUtils = ManagersUtils || {};

ManagersUtils.cxsContext = '/cxs/';

ManagersUtils.formMappingActionStrategies = ['alwaysSet', 'setIfMissing'];

ManagersUtils.getQueryOrDefault = function (query) {
    var scopedAndSharedQuery = {
        "condition": {
            "parameterValues": {
                "subConditions": [
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "metadata.scope",
                            "comparisonOperator": "equals",
                            "propertyValue": ManagersContext.dmfScope
                        }
                    },
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "metadata.scope",
                            "comparisonOperator": "equals",
                            "propertyValue": "systemscope"
                        }
                    }
                ],
                "operator": "or"
            },
            "type": "booleanCondition"
        },
        "limit": 50,
        "offset": 0,
        "sortby": null,
        "forceRefresh": false
    };

    return query || scopedAndSharedQuery;
};

ManagersUtils.parseErrorsInForm = function (form, parsedErrors, subFormName) {
    if(!parsedErrors) {
        parsedErrors = {};
    }

    angular.forEach(form.$error, function(value, key) {

        if(value.length) {
            angular.forEach(value, function(subForm) {
                ManagersUtils.parseErrorsInForm(subForm, parsedErrors, form.$name);
            });
        } else if(value && form.$name) {
            parsedErrors["wem_validation." + (subFormName.indexOf('_') != -1 ? subFormName.substr(0, subFormName.indexOf('_')) : subFormName) + "." + form.$name + "." + key] = true;
        }
    });

    return parsedErrors;
};

ManagersUtils.displayErrors = function (errorKeys, i18nService, notificationService) {
    angular.forEach(errorKeys, function(value, key) {
        notificationService.error(i18nService.message(key));
    });
};

ManagersUtils.hasErrors = function(rootForm, form, propertyName) {
    if(rootForm) {
        if(!rootForm.showErrors) {
            return false;
        }

        if(form[propertyName]) {
            return form[propertyName].$invalid;
        }
    }
    return false;
};

ManagersUtils.firstlineChartSettings = {
    fillColor: "rgba(220,220,220,0.2)",
    strokeColor: "rgba(220,220,220,1)",
    pointColor: "rgba(220,220,220,1)",
    pointStrokeColor: "#fff",
    pointHighlightFill: "#fff",
    pointHighlightStroke: "rgba(220,220,220,1)"
};

ManagersUtils.secondlineChartSettings = {
    fillColor: "rgba(0, 188, 212, 0.2)",
    strokeColor: "#00bcd4",
    pointColor: "#00bcd4",
    pointStrokeColor: "#fff",
    pointHighlightFill: "#fff",
    pointHighlightStroke: "#00bcd4"
};

ManagersUtils.getDoughnutColor = function (percentage) {
    if (percentage > 90) {
        return "#8454c6";
    }
    if (percentage > 20) {
        return "#4BE45F";
    }
    if (percentage > 5) {
        return "#ffc500";
    }
    return "#E44759";
};

ManagersUtils.goalTags = ["downloadGoal", "formGoal", "funnelGoal", "landingPageGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.campaingTags = ["downloadGoal", "formGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.optimizationTestGoalTags = ["downloadGoal", "formGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.buildTags = function (tags, i18nService) {
    var result = {};
    tags.forEach(function (tag) {
        result[tag] = {
            id: tag,
            name: i18nService.message("serverSettings.manageGoals.tag." + tag),
            description: i18nService.message("serverSettings.manageGoals.tag." + tag + ".description"),
            goals: []
        }
    });

    return result;
};

ManagersUtils.supportedTimelineIntervales = [
    {
        "key": "1M",
        "i18n": "serverSettings.manageGoals.interval.1M",
        "unitcount": 1,
        "units": "months",
        "unit": "month",
        minpoints: 6,
        maxpoints: 36,
        scaleFormat: "YYYY-MM",
        pickerFormat: "MMMM yyyy"
    },
    {
        "key": "1d",
        "i18n": "serverSettings.manageGoals.interval.1d",
        "unitcount": 1,
        "units": "days",
        "unit": "day",
        minpoints: 10,
        maxpoints: 60,
        scaleFormat: "L",
        pickerFormat: "mediumDate"
    }
];

ManagersUtils.supportedTimelineIntervalesObj = {
    "1M": ManagersUtils.supportedTimelineIntervales[0],
    "1d": ManagersUtils.supportedTimelineIntervales[1]
};

ManagersUtils.generateId = function () {
    // Math.random should be unique because of its seeding algorithm.
    // Convert it to base 36 (numbers + letters), and grab the first 9 characters
    // after the decimal.
    return '_' + Math.random().toString(36).substr(2, 9);
};
/**
 * Utility method to read nested objects in a json obj without have to test all the object in the path
 * exemple before to access goal.metadata.type you have to check if goal and metadata are not undefined before trying access type, if not you will have
 * "cannot read property type of undefined"
 *
 * Now you can use ManagersUtils.getProprety(goal, "metadata.type") safely, can return undefined in case of undefined are found in the given path
 * @param obj
 * @param propPath
 * @returns {*}
 */
ManagersUtils.getProprety = function (obj, propPath) {
    props = propPath.split('.');
    if (props.length == 1) {
        return obj[props[0]]
    } else if (props.length > 1) {
        var prop = props.shift();
        var resolvedObj = obj[prop];
        if (resolvedObj) {
            return this.getProprety(resolvedObj, props.join("."))
        }
    }
    return undefined;
};

ManagersUtils.acceptProperty = function (data, prop) {
    return (prop && data.hasOwnProperty(prop) && prop != "_all" && prop != "_filtered");
};

ManagersUtils.getCurrentModuleResource = function (resourcePath) {
    if (ManagersContext.currentModuleURL) {
        return ManagersContext.currentModuleURL + resourcePath;
    } else {
        return "/modules/marketing-factory-core" + resourcePath;
    }
};

ManagersUtils.capitalize = function (str) {
    return str.charAt(0).toUpperCase() + str.slice(1);
};

/**
 * format date in yyyy-mm
 * @param date
 * @returns {string}
 */
ManagersUtils.formatDate = function (date) {
    var yyyy = date.getFullYear().toString();
    var mm = (date.getMonth() + 1).toString(); // getMonth() is zero-based

    return yyyy + '-' + (mm[1] ? mm : "0" + mm[0]);
};


ManagersUtils.getPercentage = function (value) {
    return +((value * 100).toFixed(2)); // convert formatted string back to a number
};

ManagersUtils.downloadForm = function (url, data, method) {
    //url and data options required
    if (url && data) {
        //split params into form inputs
        var $query = jQuery('<input type="hidden" name="query"/>').val(data);
        //send request
        jQuery('<form action="' + url + '" method="' + (method || 'post') + '"></form>').append($query)
            .appendTo('body').submit().remove();
    }
};

ManagersUtils.maxInterestsInChart = 16;

ManagersUtils.getProfileDisplayName = function (i18nService, properties) {
    if (properties.firstName && properties.lastName) {
        return properties.firstName + ' ' + properties.lastName;
    } else if (properties.firstName) {
        return properties.firstName;
    } else if (properties.lastName) {
        return properties.lastName;
    } else {
        return i18nService.message("serverSettings.manageProfiles.profile.unnamed");
    }
};

ManagersUtils.campaignPropertiesForColName = {
    "id": "metadata.id",
    "name": "metadata.name",
    "description": "metadata.description",
    "startDate": "startDate",
    "endDate": "endDate"
};

ManagersUtils.initPagerProperties = function (DataTable, $scope, i18nService) {
    $scope.pageInfo = DataTable.page.info();
    $scope.pageLength = DataTable.page.len();
    $scope.pagerPages = [];

    if ($scope.pageInfo.pages < 8) {
        for (var i = 0; i < $scope.pageInfo.pages; i++) {
            $scope.pagerPages.push({
                id: i,
                label: i + 1,
                active: $scope.pageInfo.page == i,
                enabled: true
            });
        }
    } else {
        $scope.pagerPages.push({
            id: 0,
            label: 1,
            active: $scope.pageInfo.page == 0,
            enabled: true
        });
        if ($scope.pageInfo.page < 4) {
            for (var i = 1; i < 5; i++) {
                $scope.pagerPages.push({
                    id: i,
                    label: i + 1,
                    active: $scope.pageInfo.page == i,
                    enabled: true
                });
            }
        }
        $scope.pagerPages.push({
            id: 0,
            label: "...",
            active: false,
            enabled: false
        });
        if ($scope.pageInfo.page >= 4 && $scope.pageInfo.page <= $scope.pageInfo.pages - 5) {
            for (var i = $scope.pageInfo.page - 1; i <= $scope.pageInfo.page + 1; i++) {
                $scope.pagerPages.push({
                    id: i,
                    label: i + 1,
                    active: $scope.pageInfo.page == i,
                    enabled: true
                });
            }
            $scope.pagerPages.push({
                id: 0,
                label: "...",
                active: false,
                enabled: false
            });
        }

        if ($scope.pageInfo.page > $scope.pageInfo.pages - 5) {
            for (var i = $scope.pageInfo.pages - 5; i < $scope.pageInfo.pages - 1; i++) {
                $scope.pagerPages.push({
                    id: i,
                    label: i + 1,
                    active: $scope.pageInfo.page == i,
                    enabled: true
                });
            }
        }

        $scope.pagerPages.push({
            id: $scope.pageInfo.pages - 1,
            label: $scope.pageInfo.pages,
            active: $scope.pageInfo.page == $scope.pageInfo.pages - 1,
            enabled: true
        });
    }
    $scope.pagerActive = $scope.pageInfo.pages > 1;

    if (i18nService && $scope.pagerActive) {
        $scope.$watch("pageInfo.start", function (newValue, oldValue) {
            $scope.pageInfoText = i18nService.format("serverSettings.manageWem.pageInfos", $scope.pageInfo.start + 1 + "|" + $scope.pageInfo.end + "|" +
                $scope.pageInfo.recordsTotal);
        });
    }

    $scope.goToPage = function (page) {
        DataTable.page(page).draw(false);
    };

};

ManagersUtils.notificationProvider;

ManagersUtils.personaMap = {
    center: {
        latitude: 45,
        longitude: -73
    },
    zoom: 8,
    control: {},
    options:{
        draggable:true,
        styles:
            [
                {
                    "featureType": "administrative",
                    "elementType": "labels.text.fill",
                    "stylers": [
                        {
                            "color": "#444444"
                        }
                    ]
                },
                {
                    "featureType": "landscape",
                    "elementType": "all",
                    "stylers": [
                        {
                            "color": "#f2f2f2"
                        }
                    ]
                },
                {
                    "featureType": "poi",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "poi.park",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "road",
                    "elementType": "all",
                    "stylers": [
                        {
                            "saturation": -100
                        },
                        {
                            "lightness": 45
                        }
                    ]
                },
                {
                    "featureType": "road.highway",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "simplified"
                        }
                    ]
                },
                {
                    "featureType": "road.arterial",
                    "elementType": "labels.icon",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "transit",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "transit.line",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.airport",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.airport",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.bus",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.bus",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.rail",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.rail",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "water",
                    "elementType": "all",
                    "stylers": [
                        {
                            "color": "#6e769e"
                        },
                        {
                            "visibility": "on"
                        }
                    ]
                }
            ]
    }
};
