angular.module('wemServices', ['underscore', 'ui-notification', 'i18n'])

    .config(['$httpProvider', 'NotificationProvider', function ($httpProvider, NotificationProvider) {
        $httpProvider.defaults.headers.common['Accept-Language'] = ManagersContext.jahiaUILocale;

        $httpProvider.interceptors.push(function ($q) {
            return {
                'responseError': function (rejection) {
                    if (ManagersUtils.notificationProvider && rejection.status != 404) {
                        ManagersUtils.notificationProvider.error("Error : " + rejection.statusText);
                    }
                    return $q.reject(rejection);
                }
            };
        });

        NotificationProvider.setOptions({
            delay: 5000,
            startTop: 20,
            startRight: 20,
            verticalSpacing: 20,
            horizontalSpacing: 20,
            positionX: 'right',
            positionY: 'bottom'
        });
    }])

    .run(['Notification', '$rootScope', '$window', '$location',
        function (Notification, $rootScope, $window, $location) {
            ManagersUtils.notificationProvider = Notification;

            $rootScope.$on('$locationChangeSuccess', function (event) {
                $window.parent.location.hash = "#" + $location.path();
            });
        }])

    .service('notificationService', ['Notification', function (Notification) {
        return Notification;
    }])

    .service('loadingSpinnerService', function () {
        this.register = function (spinner) {
            this.spinner = spinner;
        };

        this.show = function () {
            if (this.spinner) {
                this.spinner.show();
            }
        };

        this.hide = function () {
            if (this.spinner) {
                this.spinner.hide();
            }
        }
    })

    .service('filteringService', function (_) {
        this.filterItems = function (items, service) {
            if (service.deletedItem) {
                items = _.filter(items, function (item) {
                    return item.id != service.deletedItem
                })
            }
            if (service.newItem) {
                var found = false;
                items = _.map(items, function (item) {
                    if (item.id == service.newItem.metadata.id) {
                        found = true;
                        return service.newItem.metadata;
                    } else {
                        return item;
                    }
                });
                if (!found) {
                    items.push(service.newItem.metadata);
                }
            }
            delete service.newItem;
            delete service.deletedItem;

            return items
        }
    })

    .service('goalService', ['$http', function ($http) {
        var goalsContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'goals/';

        this.getAll = function () {
            return $http.post(goalsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.getAllForSite = function () {
            return $http.post(goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "missing"
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.getAllForCampaign = function (campaign) {
            return $http.post(goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaign
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.setGoal = function (goal) {
            this.newItem = goal;
            return $http.post(goalsContext, goal);
        };

        this.getGoal = function (id) {
            return $http.get(goalsContext + id);
        };

        this.getGoalReport = function (id) {
            var uri = goalsContext + id + "/report";
            return $http.get(uri);
        };

        this.getGoalReportAggregated = function (id, query) {
            var uri = goalsContext + id + "/report";
            return $http.post(uri, query);
        };

        this.deleteGoal = function (id) {
            this.deletedItem = id;
            return $http.delete(goalsContext + id);
        };
    }])

    .service('ruleService', ['$http', function ($http) {
        var rulesContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'rules/';

        this.getAll = function () {
            return $http.post(rulesContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.setRule = function (rule) {
            return $http.post(rulesContext, rule);
        };

        this.getRule = function (id) {
            return $http.get(rulesContext + id);
        };

        this.deleteRule = function (id) {
            return $http.delete(rulesContext + id);
        }
    }])

    .service('segmentService', ['$http', 'jcrService', function ($http, jcrService) {
        var segmentsContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'segments/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedItem;
            }

            return query;
        };

        this.getAll = function () {
            return $http.get(segmentsContext);
        };

        this.getImpacted = function (segmentId) {
            return $http.get(segmentsContext + segmentId + "/impacted");
        };

        this.query = function (query) {
            return $http.post(segmentsContext + "query", query);
        };

        this._getSubConditionsOfType = function (rootCondition, list, conditionType) {
            var instance = this;
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    instance._getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        };

        this.addDownloadMixin = function(condition) {
            var instance = this;
            pastEventConditions = this._getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = instance._getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).success(function (data) {
                        // mixin added
                    });
                })
            });
        };

        this.setSegment = function (segment, shared) {
            this.addDownloadMixin(segment.condition);
            if (shared) {
                segment.metadata.scope = 'systemscope';
            }
            this.newItem = segment;
            return $http.post(segmentsContext, segment);
        };

        this.getSegment = function (id) {
            return $http.get(segmentsContext + id);
        };

        this.getSegmentCount = function (id) {
            return $http.get(segmentsContext + id + "/count");
        };

        this.deleteSegment = function (id, validate) {
            var uri = segmentsContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        }
    }])

    .service('listService', ['$http', function ($http) {
        var listsContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'lists/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedItem;
            }

            return query;
        };

        this.getAll = function () {
            return $http.get(listsContext);
        };

        this.query = function (query) {
            var query = ManagersUtils.getQueryOrDefault(query);
            return $http.post(listsContext + "query", query);
        };

        this.setList = function (list, shared) {
            if (shared) {
                list.metadata.scope = 'systemscope';
            }
            this.newItem = list;
            return $http.post(listsContext, list);
        };

        this.getList = function (id) {
            return $http.get(listsContext + id);
        };

        this.deleteList = function (id) {
            this.deletedItem = id;
            return $http.delete(listsContext + id);
        };
    }])

    .service('queryService', ['$http', 'i18nService', 'notificationService', function ($http, i18nService, notificationService) {
        var queryContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'query/';

        this.query = function (type, property, aggregate) {
            return $http.post(queryContext + type + '/' + property, aggregate);
        };

        this.getMetricsForCondition = function (type, property, metrics, condition) {
            return $http.post(queryContext + type + '/' + property + '/' + ($.isArray(metrics) ? metrics.join('/') : metrics), condition);
        };

        this.getMatchingItemsCountForCondition = function (type, condition, validate) {
            return $http.post(queryContext + type + '/count' + (validate != undefined ? ("?validate="+validate) : ""), condition);
        };

        this.createWithCallbackIfDoesNotExist = function (itemType, scope, name, typeId, errorMessageKey, errorMessageParams, callback, failCallback) {

            var condition = {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.scope",
                                comparisonOperator: "equals",
                                propertyValue: scope ? scope : ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.name",
                                comparisonOperator: "equals",
                                propertyValue: name
                            }
                        }
                    ],
                    operator: "and"
                }
            };

            if (typeId) {
                condition.parameterValues.subConditions.push({
                    type: "sessionPropertyCondition",
                    parameterValues: {
                        propertyName: "metadata.tags",
                        comparisonOperator: "contains",
                        propertyValue: typeId
                    }
                });
            }

            this.getMatchingItemsCountForCondition(itemType, condition).success(function (data) {
                // if we have matching elements, display an error
                if (data > 0) {
                    if (failCallback) {
                        failCallback();
                    }
                    notificationService.error(errorMessageParams ? i18nService.format(errorMessageKey, errorMessageParams) : i18nService.message(errorMessageKey));
                } else {
                    // otherwise perform the provide callback
                    callback();
                }
            });

        };

        this.nbOfVisits = function (campaignId, startDate, endDate, callback) {
            this.getMetricsForCondition("session", "nbOfVisits", ["sum", "avg"], {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "scope",
                                "comparisonOperator": "equals",
                                "propertyValue": ManagersContext.dmfScope
                            }
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "systemProperties.campaigns." + campaignId + "Engaged",
                                "comparisonOperator": "between",
                                "propertyValuesDate": [startDate, endDate]
                            }
                        }
                    ]
                }
            }).success(callback);
        };
    }])

    .service('definitionService', ['$http', function ($http) {
        var definitionsContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'definitions/';

        this.getTag = function (tag) {
            return $http.get(definitionsContext + 'tags/' + tag)
        };

        this.getConditionTypes = function (tag) {
            return $http.get(definitionsContext + 'conditions')
        };

        this.getConditionType = function (conditionType) {
            return $http.get(definitionsContext + 'conditions/' + conditionType)
        };

        this.getConditionTypeByTag = function (tag) {
            return $http.get(definitionsContext + 'conditions/tags/' + tag)
        };

        this.fillTags = function (arrayName, $scope) {
            $scope.$watch(arrayName + ".length", function (newVal, oldVal, scope) {
                angular.forEach($scope[arrayName], function (condition) {
                    angular.forEach(condition.tags, function (tag) {
                        if ($scope.tags[tag]) {
                            condition.profileTag = tag;
                        }
                    });
                });
            });
        }
    }])

    .service('profileService', ['$http', function ($http) {
        var profilesContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'profiles/';
        var personasContext = profilesContext + 'personas/';

        var getContextFor = function (profileType) {
            return profileType == 'personas' ? personasContext : profilesContext;
        };

        this.search = function (query, profileType) {
            return $http.post(getContextFor(profileType) + "search", query);
        };

        this.exportUsingForm = function (query, profileType, method) {
            ManagersUtils.downloadForm(getContextFor(profileType) + "export", query, method);
        };

        this.getProfile = function (id, profileType) {
            return $http.get(getContextFor(profileType) + id);
        };

        this.saveProfile = function (profile, profileType) {
            // remove empty props
            var cleanedProp = {};
            angular.forEach(profile.properties, function (value, key) {
                if (value) {
                    cleanedProp[key] = value;
                }
            });
            profile.properties = cleanedProp;
            return $http.post(getContextFor(profileType), profile);
        };

        this.deleteProfile = function (id, profileType) {
            return $http.delete(getContextFor(profileType) + id);
        };

        this.newProfile = function (id, profileType) {
            return $http.put(getContextFor(profileType) + id);
        };

        this.batchProfilesUpdate = function (batchUpdate) {
            return $http.post(profilesContext + 'batchProfilesUpdate', batchUpdate)
        };

        this.getSessions = function (id, profileType, offset, numberOfSessions, sort) {
            return $http.get(getContextFor(profileType) + id + '/sessions?offset=' + offset + '&size=' + numberOfSessions + '&sort=' + sort);
        };

        this.getPropertyTypes = function () {
            return $http.get(profilesContext + 'properties');
        };

        this.createPropertyType = function (propertyType) {
            return $http.post(profilesContext + 'properties', propertyType);
        };

        this.deletePropertyType = function (propertyId) {
            return $http.delete(profilesContext + 'properties/' + propertyId);
        };

        this.getPropertyTypesByTarget = function (target) {
            return $http.get(profilesContext + 'properties/' + target)
        };
        this.getPropertyTypesByTag = function (tag, recursive) {
            if (recursive) {
                return $http.get(profilesContext + 'properties/tags/' + tag + '?recursive=true')
            } else {
                return $http.get(profilesContext + 'properties/tags/' + tag)
            }
        };

        this.getCount = function () {
            return $http.get(profilesContext + 'count')
        };


        this.constructCondition = function (baseCondition, searchCondition, deletedProfileId, excludeMerged, profileType) {
            var subConditions = [];
            if (baseCondition) {
                subConditions.push(baseCondition);
            }
            if (excludeMerged && profileType != "personas") {
                // we do not want the merged profiles:
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "missing",
                        propertyName: "mergedWith"
                    },
                    type: "profilePropertyCondition"
                });
            }
            if (searchCondition && searchCondition.type) {
                if (searchCondition.type == "booleanCondition"
                    && searchCondition.parameterValues.operator
                    && searchCondition.parameterValues.operator == "and"
                    && searchCondition.parameterValues.subConditions) {
                    subConditions = subConditions.concat(searchCondition.parameterValues.subConditions);
                } else {
                    subConditions.push(searchCondition);
                }
            }
            if (deletedProfileId) {
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: deletedProfileId
                    },
                    type: "profilePropertyCondition"
                });
            }

            if (subConditions.length == 0) {
                return null;
            } else if (subConditions.length == 1) {
                return subConditions[0];
            } else {
                return {
                    parameterValues: {
                        operator: "and",
                        subConditions: subConditions
                    },
                    type: "booleanCondition"
                };
            }
        };

        this.constructQuery = function (baseCondition, searchCondition, deletedProfileId, text, offset, limit, orderby, profileType) {
            return {
                "condition": this.constructCondition(baseCondition, searchCondition, deletedProfileId, true, profileType),
                "limit": limit ? limit : 50,
                "offset": offset ? offset : 0,
                "sortby": orderby != undefined ? orderby : null,
                "text": text != undefined ? text : null
            };
        };

        this.constructQueryOnCurrentSelectedTab = function (text, offset, limit, orderby) {
            if (this.selectedTab) {
                return {
                    "condition": this.constructCondition(this.selectedTab.baseCondition, this.selectedTab.searchCondition, this.selectedTab.deletedProfileId, true, this.selectedTab.type),
                    "limit": limit ? limit : -1,
                    "offset": offset ? offset : 0,
                    "sortby": orderby != undefined ? orderby : null,
                    "text": text != undefined ? text : null
                };
            }
        };

    }])

    .service('jcrService', ['$http', function ($http) {
        this.buildURL = function (workspace, locale, type, endUrl) {
            return [ManagersContext.jcrRestAPIBase,
                    workspace ? workspace : "default",
                    locale ? locale : ManagersContext.jahiaUILocale,
                    type ? type : "nodes"].join("/") + endUrl;
        };

        this.addMixin = function (workspace, locale, nodeIdOrPath, mixin, mixinProperties, byPath) {
            var apiType = byPath ? "paths" : "nodes";
            nodeIdOrPath = byPath ? nodeIdOrPath : "/" + nodeIdOrPath;
            var url = this.buildURL(workspace, locale, apiType, nodeIdOrPath) + "/mixins/" + mixin.replace(":", "__");
            return mixinProperties ? $http.put(url, mixinProperties) : $http.put(url);
        };

        this.doGetOnPath = function (workspace, locale, path) {
            return $http.get(this.buildURL(workspace, locale, "paths", path));
        };

        this.doGetOnPathSync = function (workspace, locale, path) {
            var request = new XMLHttpRequest();
            request.open('GET', this.buildURL(workspace, locale, "paths", path), false);
            request.send(null);

            return request.status === 200;
        }
    }])

    .service('campaignService', ['$http', 'i18nService', function ($http, i18nService) {
        var campaignsContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'campaigns/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (this.deletedCampaign) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedCampaign
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedCampaign
            }

            return query;
        };

        this.getAll = function () {
            return $http.post(campaignsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.queryDetails = function (query) {
            return $http.post(campaignsContext + "query/detailed", query);
        };

        this.setCampaign = function (campaign) {
            return $http.post(campaignsContext, campaign);
        };

        this.setCampaignEvent = function (campaignEvent) {
            this.newEvent = campaignEvent;
            return $http.post(campaignsContext + "event", campaignEvent);
        };

        this.getCampaign = function (id) {
            return $http.get(campaignsContext + id);
        };

        this.getCampaignDetail = function (id) {
            return $http.get(campaignsContext + id + "/detailed");
        };

        this.getCampaignType = function (campaign) {
            return campaign.metadata.tags[0];
        };

        this.getLocalizedCampaignType = function (campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType);
        };

        this.getLocalizedCampaignParam = function (campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType + '.paramName');
        };

        this.getTypeSpecificCampaignParameter = function (campaign) {
            var campaignType = this.getCampaignType(campaign);
            switch (campaignType) {
                case 'landing':
                    return campaign.entryCondition.parameterValues.subConditions[1].parameterValues.pagePath;
                case 'referrer':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
                case 'parameter':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
            }
        };

        this.getCampaignEvents = function (campaignId, length, start, order, additionalCondition) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaignId
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (additionalCondition) {
                query.condition.parameterValues.subConditions.push(additionalCondition);
            }

            if (this.newEvent) {
                query.forceRefresh = true;
                delete this.newEvent;
            }

            if (this.deletedEvent) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedEvent
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedEvent;
            }
            return $http.post(campaignsContext + "events/query", query);
        };

        this.deleteCampaign = function (id) {
            this.deletedCampaign = id;
            return $http.delete(campaignsContext + id);
        };

        this.deleteCampaignEvent = function (eventId) {
            this.deletedEvent = eventId;
            return $http.delete(campaignsContext + "event/" + eventId);
        }
    }])

    .service('formMappingService', ['$http', 'i18nService', 'loadingSpinnerService', function ($http, i18nService, loadingSpinnerService) {
        var formMappingContext = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'rules/';
        var openMappedProp = "(";
        var setPropertyNamePrefix = "properties" + openMappedProp;
        var setPropertyValuePrefix = "eventProperty::properties" + openMappedProp;
        var endPrefix = ")";
        this.save = function (formMapping) {
            this.newItem = formMapping;
            return $http.post(formMappingContext, formMapping);
        };

        this.getAll = function (offset, limit, sortby) {

            var query = {
                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.tags",
                                    "comparisonOperator": "equals",
                                    "propertyValue": "formMappingRule"
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ]
                    }
                },
                "offset": offset,
                "limit": limit,
                "sortby": sortby
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            query.condition,
                            {
                                parameterValues: {
                                    comparisonOperator: "notEquals",
                                    propertyName: "itemId",
                                    propertyValue: this.deletedItem
                                },
                                type: "sessionPropertyCondition"
                            }
                        ]
                    }
                };
                delete this.deletedItem;
            }

            return $http.post(formMappingContext + "query", query);
        };

        this.get = function (id) {
            return $http.get(formMappingContext + id);
        };

        this.getFormInPage = function ($scope, formMapping, path, callback) {
            var $iframe = $("<iframe id='formMappingFrame'></iframe>");
            $iframe.css("display", "none");
            $("body").append($iframe);
            loadingSpinnerService.show();
            $iframe.ready(function () {
                window.setTimeout(function () {
                    $scope.$apply(function() {
                        var _form = $iframe.contents().find("form[name='" + formMapping.formId + "']");
                        if (!_form || _form.length == 0) {
                            _form = $iframe.contents().find("#" + formMapping.formId);
                        }
                        $("#formMappingFrame").remove();
                        loadingSpinnerService.hide();
                        return callback(_form);
                    });
                }, 2000);
            });
            $iframe.attr("src", ManagersContext.baseLive + path + ".html?skipWem=true");
        };

        this.serverToLocalFormMapping = function (formMapping) {
            var _formMapping = {
                name: formMapping.metadata.name,
                description: formMapping.metadata.description,
                id: formMapping.metadata.id,
                scope: formMapping.metadata.scope,
                additionalPaths: [],
                tempActions: formMapping.actions,
                actions: {},
                lists: []
            };

            _formMapping.formId = _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type == 'formEventCondition';
            }).parameterValues.formId;

            _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type != 'formEventCondition';
            }).parameterValues.subConditions.forEach(function (entry) {
                    if (!_formMapping.primaryPath) {
                        _formMapping.primaryPath = entry.parameterValues.path;
                    } else {
                        _formMapping.additionalPaths.push(entry.parameterValues.path);
                    }
                });

            return _formMapping;
        };

        this.serverToLocalFormMappingActions = function (formMapping, propertyNameChoiceListValuesIndexById) {

            function extractNameFromPrefixedName(prefixedName, prefix) {
                var extracted;
                if (prefixedName.indexOf(endPrefix) > 0) {
                    extracted = prefixedName.substring(prefix.length, prefixedName.length - endPrefix.length);
                } else {
                    // we have an old style property, so remove the old style prefix which wasn't using indexed properties
                    // prefix contains the opening of the mapped prop ((') so remove it and add + 1 for the . that was there before
                    extracted = prefixedName.substr(prefix.length - openMappedProp.length + 1);
                }
                return extracted;
            }

            formMapping.tempActions.forEach(function (action) {
                if (action.parameterValues.setPropertyName == "systemProperties.lists" && action.parameterValues.setPropertyValue) {
                    formMapping.lists.push(action.parameterValues.setPropertyValue);
                } else {
                    var propertyName = extractNameFromPrefixedName(action.parameterValues.setPropertyName, setPropertyNamePrefix);
                    var fieldName = action.parameterValues.setPropertyValue ? extractNameFromPrefixedName(action.parameterValues.setPropertyValue, setPropertyValuePrefix) : undefined;
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueInteger ? extractNameFromPrefixedName(action.parameterValues.setPropertyValueInteger, setPropertyValuePrefix) : undefined;
                    }
                    var strategy = action.parameterValues.setPropertyStrategy ? action.parameterValues.setPropertyStrategy : undefined;
                    if (!propertyNameChoiceListValuesIndexById[propertyName]) {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById["user-specified-property"],
                            newProp: propertyName,
                            strategy: strategy
                        };
                    } else {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById[propertyName],
                            strategy: strategy
                        };
                    }
                }
            });
            delete formMapping.tempActions;
        };

        this.localToServerFormMapping = function (formMapping) {
            var _formMapping = {
                "metadata": {
                    "id": formMapping.id,
                    "name": formMapping.name,
                    "description": formMapping.description,
                    "scope": formMapping.scope,
                    "tags": ['formMappingRule']
                },

                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": []
                    }
                },

                "actions": []
            };

            //conditions
            var pathConditions = [];
            pathConditions.push({
                "parameterValues": {
                    "path": formMapping.primaryPath
                },
                "type": "sourceEventPropertyCondition"
            });
            formMapping.additionalPaths.forEach(function (path) {
                pathConditions.push({
                    "parameterValues": {
                        "path": path
                    },
                    "type": "sourceEventPropertyCondition"
                })
            });

            _formMapping.condition.parameterValues.subConditions.push(
                {
                    "parameterValues": {
                        "formId": formMapping.formId
                    },
                    "type": "formEventCondition"
                },
                {
                    "parameterValues": {
                        "operator": "or",
                        "subConditions": pathConditions
                    },
                    "type": "booleanCondition"
                }
            );

            function prefixPropertyName(name, prefix) {
                return prefix + name + endPrefix;
            }

            //actions
            angular.forEach(formMapping.actions, function(value) {
                value.order = ManagersUtils.formMappingActionStrategies.length - ManagersUtils.formMappingActionStrategies.indexOf(value.strategy);
            });

            angular.forEach(_.sortBy(formMapping.actions, 'order'), function (value) {
                if (value.propertyType) {
                    var strategy = value.strategy;
                    var target = ((value.propertyType.metadata.id == 'user-specified-property' && value.newProp) ? value.newProp : value.propertyType.metadata.id);
                    var action = {
                        "parameterValues": {
                            "setPropertyName": prefixPropertyName(target, setPropertyNamePrefix),
                            "setPropertyStrategy": strategy
                        },
                        "type": "setPropertyAction"
                    };

                    if (value.propertyType.type == "integer") {
                        action.parameterValues.setPropertyValueInteger = prefixPropertyName(value.fieldName, setPropertyValuePrefix);
                    } else {
                        action.parameterValues.setPropertyValue = prefixPropertyName(value.fieldName, setPropertyValuePrefix);
                    }
                    _formMapping.actions.push(action);
                }
            });

            //lists
            angular.forEach(formMapping.lists, function (value, key) {
                _formMapping.actions.push({
                    "parameterValues": {
                        "setPropertyName": "systemProperties.lists",
                        "setPropertyValue": value,
                        "setPropertyStrategy": "addValue"
                    },
                    "type": "setPropertyAction"
                });
            });

            return _formMapping;
        };

        this.remove = function (id) {
            this.deletedItem = id;
            return $http.delete(formMappingContext + id);
        }

    }])

    .service('internalSearchService', ['$http', 'i18nService', 'queryService', function ($http, i18nService, queryService) {
        var Search = function (searchString, occurrences) {
            var SearchItem = function (value, occurrences) {
                this.item = value;
                this.occurrences = occurrences;

                this.importance = function (totalOccurrences) {
                    return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
                }
            };

            this.searchString = searchString;
            this.occurrences = occurrences;

            this.origins = [];
            this.setOrigins = function (origins) {
                this.origins = this._setDetails(origins);
            };

            this.userIds = [];
            this.setUserIds = function (userIds) {
                this.userIds = this._setDetails(userIds);
            };

            this.importance = function (totalOccurrences) {
                return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
            };

            this._setDetails = function (details) {
                if (details) {
                    var detailsArray = [];
                    var totalOccurrences = 0;
                    for (var detail in details) {
                        if (ManagersUtils.acceptProperty(details, detail)) {
                            occurrences = details[detail];

                            if (detail == "_missing") {
                                // replace _missing by something more user friendly
                                detail = i18nService.message("serverSettings.internalSearches.missing");
                            }

                            totalOccurrences += occurrences;
                            detailsArray.push(new SearchItem(detail, occurrences));
                        }
                    }
                    this.occurrences = totalOccurrences;
                    return detailsArray;
                }
            };

            this.languages = [];
            this.setLanguages = function (lang) {
                this.languages = this._setDetails(lang);
            }
        };

        var searchEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "search"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };


        this.selectSearch = function (searchTerm) {
            this.selectedSearch = searchTerm;
        };


        this.getSearchOccurrencesByTerm = function () {
            return queryService.query('event', 'keyword', searchEventsCondition);
        };

        this.getSearchDetails = function (searchTerm) {
            // if we didn't provide a search term, use the one potentially already selected
            searchTerm = searchTerm || this.selectedSearch;

            var search = new Search(searchTerm, -1);

            var condition = {};
            angular.copy(searchEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "keyword",
                    "comparisonOperator": "equals",
                    "propertyValue": searchTerm
                }
            });

            queryService.query('event', 'properties.origin', condition).success(function (data) {
                search.setOrigins(data);
            });

            queryService.query('event', 'profileId', condition).success(function (data) {
                search.setUserIds(data);
            });

            queryService.query('event', 'properties.language', condition).success(function (data) {
                search.setLanguages(data);
            });

            return search;
        }
    }])
    .service('geonamesService', ['$http', function ($http) {
        this.getEntries = function () {
            var url = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'geonames/entries';
            var length = url.length;
            if (arguments.length > 0) {
                for (var i = 0; i < arguments.length; i++) {
                    url = url + ("/" + arguments[i]);
                }
            }
            if (length == url.length) {
                url += "/";
            }

            return $http.get(url);
        };

        this.getHierarchy = function (administrativeSbudivision) {
            return $http.get(ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'geonames/hierarchy/' + administrativeSbudivision);
        };

        this.getCities = function () {
            var url = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'geonames/cities';
            var length = url.length;
            if (arguments.length > 0) {
                for (var i = 0; i < arguments.length; i++) {
                    if (arguments[i]) {
                        url = url + ("/" + arguments[i]);
                    }
                }
            }
            if (length == url.length) {
                url += "/";
            }
            return $http.get(url);
        };

        this.getCapital = function (itemId) {
            var url = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'geonames/capitals/' + itemId;
            return $http.get(url);
        };

        this.reverseGeoCode = function (lat, lon) {
            var url = ManagersContext.contextServerAdminUrl + ManagersUtils.cxsContext + 'geonames/reverseGeoCode/' + lat + ',' + lon;
            return $http.get(url);
        }
    }]);
